Imports System
Imports System.Web
Imports Microsoft.VisualBasic

Partial Class HTMLConverter
    Inherits Page

    Private Sub Page_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Demo
        If Not Page.IsPostBack Then
            Dim myHTML, myText As String
            myHTML = "<head><title>Test</title></title></head><body><style>.test { color:red;}</style>This is a HTML" & vbLf & "		 test <a href='http://test/'>Test&nbsp;Link</a><p />&#169; me<br> &raquo; "
            myText = "This is a Text test" & vbLf & vbTab & "Line 2      http://test/ End line"

            'Test the HTML to Text code
            litText.Text = "<b>Original HTML:</b><br>" & vbLf & myHTML & vbLf & "<p><b>Converted Text:</b><br>" & vbLf
            litText.Text &= HTMLText.HTML2Text(myHTML)

            'Test the Text to HTML code
            litHTML.Text = "<b>Original Text:</b><br>" & vbLf & myText & vbLf & "<p><b>Converted HTML:</b><br>" & vbLf
            litHTML.Text &= HTMLText.Text2HTML(myText)

            'Example of testing if a string contains HTML
            If HTMLText.ContainsHTML(myHTML) Then
                'Do something
            End If
        End If
    End Sub
End Class

'Copy the below class into your code...

Public Class HTMLText
    'HTMLText 2.1
    'Written by Hunter Beanland - www.beanland.net.au

    ''' <summary>
    ''' Convert HTML to text while preserving as much formatting as possible
    ''' </summary>
    ''' <param name="TextHTML">HTML</param>
    ''' <returns>Text</returns>
    ''' <remarks></remarks>
    Public Shared Function HTML2Text(ByVal TextHTML As String) As String
        Dim regEx As New Regex(String.Empty, RegexOptions.Multiline Or RegexOptions.IgnoreCase)
        'Remove rubbish before the body
        If InStr(1, TextHTML, "<body", CompareMethod.Text) > 0 Then TextHTML = Right(TextHTML, TextHTML.Length - InStr(1, TextHTML, "<body", CompareMethod.Text) + 1)
        'Remove space/tabs
        TextHTML = regEx.Replace(TextHTML, "\s+", " ")
        'Insert new formatting
        TextHTML = regEx.Replace(TextHTML, "<br[^>]*>|<p[^>]*>|<div[^>]*>|<tr[^>]*>|<li[^>]*>", vbCrLf)
        TextHTML = regEx.Replace(TextHTML, "<td[^>]*>", vbTab)
        'Remove style and script structures.
        TextHTML = regEx.Replace(TextHTML, "<style[^>]*>.*</style>", String.Empty)
        TextHTML = regEx.Replace(TextHTML, "<script[^>]*>.*</script>", String.Empty)
        'Convert HTML links to Text and URL. If link are text are same, ignore text 
        TextHTML = regEx.Replace(TextHTML, "<a[^>]+href=['|""]([^'|""]*)['|""][^>]*>\1</a>", " $1 ")
        TextHTML = regEx.Replace(TextHTML, "<a[^>]+href=['|""]([^'|""]+)['|""][^>]*>(.*)</a>", " $2 $1 ")
        'Line feeds and other tags inside the anchor are sometimes missed by the last statement (and this one misses ones the last want catches)
        TextHTML = regEx.Replace(TextHTML, "<a[^>]+href=['|""]([^'|""]+)['|""][^>]*>([^<]*)", " $2 $1 ")
        'Strip remaining HTML
        TextHTML = regEx.Replace(TextHTML, "<[^>]*>", " ")
        'Replace html encoded chars (ie &#64;,&nbsp;)
        TextHTML = HttpContext.Current.Server.HtmlDecode(TextHTML)

        'Remove surplus space and line feeds & left trim (from tag removal)
        TextHTML = regEx.Replace(TextHTML, "^ +", String.Empty)
        Do While TextHTML.Contains("  ")
            TextHTML = TextHTML.Replace("  ", " ")
        Loop
        Do While TextHTML.Contains(vbCrLf & vbCrLf & vbCrLf)
            TextHTML = TextHTML.Replace(vbCrLf & vbCrLf & vbCrLf, vbCrLf & vbCrLf)
        Loop
        Return TextHTML
    End Function

    ''' <summary>
    ''' Convert Text to HTML, inserting links, paragraphs and encoding
    ''' </summary>
    ''' <param name="TextHTML">Text</param>
    ''' <returns>HTML</returns>
    ''' <remarks></remarks>
    Public Shared Function Text2HTML(ByVal TextHTML As String) As String
        Dim regEx As New Regex(String.Empty, RegexOptions.Multiline Or RegexOptions.IgnoreCase)
        'Insert spaces around link with <> or () around it before the encode so it does not become part of the link
        TextHTML = regEx.Replace(TextHTML, "([<|(])(http|ftp)(.*)([>|)])", "$1 $2$3 $4")
        'Standard encode
        TextHTML = HttpContext.Current.Server.HtmlEncode(TextHTML)
        'Replace URLs with a link
        TextHTML = regEx.Replace(TextHTML, "(http|ftp)(\S*)", "<a href=""$1$2"">$1$2</a>")
        'Replace standard formating
        TextHTML = TextHTML.Replace(vbLf & vbLf, "<p />")
        TextHTML = TextHTML.Replace(vbCrLf, "<br />")
        TextHTML = TextHTML.Replace(vbLf, "<br />")
        TextHTML = TextHTML.Replace(vbCr, "<br />")
        TextHTML = TextHTML.Replace(vbTab, "&nbsp;&nbsp;&nbsp;")
        'Replace mutli spaces
        While TextHTML.Contains("  ")
            TextHTML = TextHTML.Replace("  ", "&nbsp;&nbsp;")
        End While
        'add decent font - remove this if you don't want it.
        regEx = Nothing
        Return "<span style=""font-family:arial, helvetica, sans-serif; font-size:small;"">" & TextHTML & "</span>"
    End Function

    ''' <summary>
    ''' Detects HTML tags. Use if you are unsure if there is HTML in your text. Not fool proof
    ''' </summary>
    ''' <param name="TextHTML">HTML/Text</param>
    ''' <returns>True if contains HTML</returns>
    ''' <remarks>If you run HTML2Text and there is no HTML in the text, then the text formatting will be lost - so use this</remarks>
    Public Shared Function ContainsHTML(ByVal TextHTML As String) As Boolean
        Dim regEx As New Regex(String.Empty, RegexOptions.Multiline Or RegexOptions.IgnoreCase)
        Return regEx.IsMatch(TextHTML, "<([^>]*)>[^<]*<\/\1>|<[^>]+/>|<br[^>]*>|<p[^>]*>|<img[^>]*>")
    End Function
End Class
